@@ -226,3 +226,30 @@ class NoFreeSetId(acos_errors.ACOSException):
     def __init__(self):
         msg = ('No free VRRP-A set_id available.')
         super(NoFreeSetId, self).__init__(msg=msg)
+
+
+class ProjectDeviceNotFound(acos_errors.ACOSException):
+    def __init__(self):
+        msg = ('No valid device found for this project, please check your configuration file.')
+        super(ProjectDeviceNotFound, self).__init__(msg=msg)
+
+
+class DeviceIsProjectDevice(acos_errors.ACOSException):
+    def __init__(self):
+        msg = ('The device already specify project_id in configuration file, '
+               'device-name flavor can not use it.')
+        super(DeviceIsProjectDevice, self).__init__(msg=msg)
+
+
+class FlavorDeviceNotFound(acos_errors.ACOSException):
+    def __init__(self, device):
+        msg = ('[device-name flavor] Device [{0}] not found in the configuration'
+               ' file.').format(device)
+        super(FlavorDeviceNotFound, self).__init__(msg=msg)
+
+
+class InterfaceNotFound(acos_errors.ACOSException):
+    def __init__(self, comput_id, network_id):
+        msg = ('vThunder instance {0} has no interface in network {1}.').format(comput_id,
+                                                                                network_id)
+        super(InterfaceNotFound, self).__init__(msg=msg)
