@@ -13,14 +13,13 @@
 # under the License.
 #
 import acos_client.errors as acos_errors
+import copy
 from neutronclient.common import exceptions as neutron_exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import uuidutils
 from requests import exceptions as req_exceptions
 import six
-import socket
-import struct
 from taskflow import task
 from taskflow.types import failure
 
@@ -368,7 +367,7 @@ class PlugVIP(BaseNetworkTask):
                     amphora.vrrp_port_id = amp_data.vrrp_port_id
                     amphora.ha_port_id = amp_data.ha_port_id
 
-            self.network_driver.unplug_vip(loadbalancer, loadbalancer.vip)
+            self.network_driver.unplug_vip_revert(loadbalancer, loadbalancer.vip)
         except Exception as e:
             LOG.error("Failed to unplug VIP.  Resources may still "
                       "be in use from vip: %(vip)s due to error: %(except)s",
@@ -683,7 +682,7 @@ class HandleVRIDFloatingIP(BaseNetworkTask):
         self.added_fip_ports = []
         super(HandleVRIDFloatingIP, self).__init__(*arg, **kwargs)
 
-    def _add_vrid_to_list(self, vrid_list, subnet, project_id):
+    def _add_vrid_to_list(self, vrid_list, subnet, owner):
         vrid_value = CONF.a10_global.vrid
         filtered_vrid_list = list(filter(lambda x: x.subnet_id == subnet.id, vrid_list))
         if not filtered_vrid_list:
@@ -691,11 +690,10 @@ class HandleVRIDFloatingIP(BaseNetworkTask):
                 data_models.VRID(
                     id=uuidutils.generate_uuid(),
                     vrid=vrid_value,
-                    project_id=project_id,
+                    owner=owner,
                     vrid_port_id=None,
                     vrid_floating_ip=None,
                     subnet_id=subnet.id))
-        return vrid_list
 
     def _remove_device_vrid_fip(self, partition_name, vrid_value):
         try:
@@ -745,7 +743,8 @@ class HandleVRIDFloatingIP(BaseNetworkTask):
         return vrid
 
     @axapi_client_decorator
-    def execute(self, vthunder, lb_resource, vrid_list, subnet):
+    def execute(self, vthunder, lb_resource, vrid_list, subnet,
+                vthunder_config, use_device_flavor=False):
         """
         :param vthunder:
         :param lb_resource: Can accept LB or member
@@ -757,13 +756,20 @@ class HandleVRIDFloatingIP(BaseNetworkTask):
         :return: return the update list of VRID object, If empty the need to remove all VRID
         objects from DB else need update existing ones.
         """
-
         vrid_value = CONF.a10_global.vrid
         prev_vrid_value = vrid_list[0].vrid if vrid_list else None
-        conf_floating_ip = a10_utils.get_vrid_floating_ip_for_project(lb_resource.project_id)
+        updated_vrid_list = copy.copy(vrid_list)
+        if use_device_flavor:
+            if vthunder_config.vrid_floating_ip:
+                conf_floating_ip = vthunder_config.vrid_floating_ip
+            else:
+                conf_floating_ip = CONF.a10_global.vrid_floating_ip
+        else:
+            conf_floating_ip = a10_utils.get_vrid_floating_ip_for_project(
+                lb_resource.project_id)
 
         if not conf_floating_ip:
-            for vrid in vrid_list:
+            for vrid in updated_vrid_list:
                 self._delete_vrid_port(vrid.vrid_port_id)
             vrid_value = prev_vrid_value if prev_vrid_value else vrid_value
             self._remove_device_vrid_fip(vthunder.partition_name, vrid_value)
@@ -771,24 +777,47 @@ class HandleVRIDFloatingIP(BaseNetworkTask):
 
         vrid_floating_ips = []
         update_vrid_flag = False
-        vrid_list = self._add_vrid_to_list(vrid_list, subnet, lb_resource.project_id)
-        for vrid in vrid_list:
-            subnet = self.network_driver.get_subnet(vrid.subnet_id)
+        existing_fips = []
+        if use_device_flavor:
+            owner = a10_utils.get_device_vrid_owner(
+                vthunder_config.device_name, vthunder_config.partition_name,
+                vthunder_config.hierarchical_multitenancy)
+            self._add_vrid_to_list(updated_vrid_list, subnet, owner)
+        else:
+            self._add_vrid_to_list(updated_vrid_list, subnet, lb_resource.project_id)
+        for vrid in updated_vrid_list:
+            try:
+                vrid_summary = self.axapi_client.vrrpa.get(vrid.vrid)
+            except Exception as e:
+                vrid_summary = {}
+                LOG.exception("Failed to get existing VRID summary due to: %s", str(e))
+
+            if vrid_summary and 'floating-ip' in vrid_summary['vrid']:
+                vrid_fip = vrid_summary['vrid']['floating-ip']
+                if vthunder.partition_name != 'shared':
+                    for i in range(len(vrid_fip['ip-address-part-cfg'])):
+                        existing_fips.append(
+                            vrid_fip['ip-address-part-cfg'][i]['ip-address-partition'])
+                else:
+                    for i in range(len(vrid_fip['ip-address-cfg'])):
+                        existing_fips.append(vrid_fip['ip-address-cfg'][i]['ip-address'])
+            vrid_subnet = self.network_driver.get_subnet(vrid.subnet_id)
             vrid.vrid = vrid_value
             if conf_floating_ip.lower() == 'dhcp':
                 subnet_ip, subnet_mask = a10_utils.get_net_info_from_cidr(
-                    subnet.cidr)
+                    vrid_subnet.cidr)
                 if not a10_utils.check_ip_in_subnet_range(
                         vrid.vrid_floating_ip, subnet_ip, subnet_mask):
-                    vrid = self._replace_vrid_port(vrid, subnet, lb_resource)
+                    vrid = self._replace_vrid_port(vrid, vrid_subnet, lb_resource)
                     update_vrid_flag = True
             else:
                 new_ip = a10_utils.get_patched_ip_address(
-                    conf_floating_ip, subnet.cidr)
+                    conf_floating_ip, vrid_subnet.cidr)
                 if new_ip != vrid.vrid_floating_ip:
-                    vrid = self._replace_vrid_port(vrid, subnet, lb_resource, new_ip)
+                    vrid = self._replace_vrid_port(vrid, vrid_subnet, lb_resource, new_ip)
                     update_vrid_flag = True
-            vrid_floating_ips.append(vrid.vrid_floating_ip)
+            if vrid_subnet.id == subnet.id or vrid.vrid_floating_ip in existing_fips:
+                vrid_floating_ips.append(vrid.vrid_floating_ip)
 
         if (prev_vrid_value is not None) and (prev_vrid_value != vrid_value):
             self._remove_device_vrid_fip(vthunder.partition_name, prev_vrid_value)
@@ -796,7 +825,7 @@ class HandleVRIDFloatingIP(BaseNetworkTask):
         elif update_vrid_flag:
             self._update_device_vrid_fip(vthunder.partition_name, vrid_floating_ips, vrid_value)
 
-        return vrid_list
+        return updated_vrid_list
 
     @axapi_client_decorator
     def revert(self, result, vthunder, lb_resource, vrid_list, subnet, *args, **kwargs):
@@ -815,11 +844,11 @@ class HandleVRIDFloatingIP(BaseNetworkTask):
 
         vrid_floating_ip_list = [vrid.vrid_floating_ip for vrid in vrid_list]
 
-        if vrid_floating_ip_list:
+        if isinstance(vrid_floating_ip_list, list):
             vrid_value = CONF.a10_global.vrid
             try:
                 self._update_device_vrid_fip(
-                    vthunder, vrid_floating_ip_list, vrid_value)
+                    vthunder.partition_name, vrid_floating_ip_list, vrid_value)
             except Exception as e:
                 LOG.error("Failed to update VRID floating IPs %s due to %s",
                           vrid_floating_ip_list, str(e))
@@ -831,15 +860,35 @@ class DeleteVRIDPort(BaseNetworkTask):
 
     @axapi_client_decorator
     def execute(self, vthunder, vrid_list, subnet,
-                lb_count_subnet, member_count, lb_resource):
+                use_device_flavor, lb_count_subnet, member_count,
+                lb_count_thunder, member_count_thunder, lb_resource):
         vrid = None
         vrid_floating_ip_list = []
-        resource_count = lb_count_subnet + member_count
+        existing_fips = []
+        if use_device_flavor:
+            resource_count = lb_count_thunder + member_count_thunder
+        else:
+            resource_count = lb_count_subnet + member_count
         if resource_count <= 1 and vthunder:
             for vr in vrid_list:
+                try:
+                    vrid_summary = self.axapi_client.vrrpa.get(vr.vrid)
+                except Exception as e:
+                    vrid_summary = {}
+                    LOG.exception("Failed to get existing VRID summary due to: %s", str(e))
+
+                if vrid_summary and 'floating-ip' in vrid_summary['vrid']:
+                    vrid_fip = vrid_summary['vrid']['floating-ip']
+                    if vthunder.partition_name != 'shared':
+                        for i in range(len(vrid_fip['ip-address-part-cfg'])):
+                            existing_fips.append(
+                                vrid_fip['ip-address-part-cfg'][i]['ip-address-partition'])
+                    else:
+                        for i in range(len(vrid_fip['ip-address-cfg'])):
+                            existing_fips.append(vrid_fip['ip-address-cfg'][i]['ip-address'])
                 if vr.subnet_id == subnet.id:
                     vrid = vr
-                else:
+                elif vr.vrid_floating_ip in existing_fips:
                     vrid_floating_ip_list.append(vr.vrid_floating_ip)
             if vrid:
                 try:
@@ -870,14 +919,31 @@ class DeleteMultipleVRIDPort(BaseNetworkTask):
                 amphorae = a10_task_utils.attribute_search(lb_resource, 'amphorae')
                 vrids = []
                 vrid_floating_ip_list = []
+                existing_fips = []
                 for vrid in vrid_list:
+                    try:
+                        vrid_summary = self.axapi_client.vrrpa.get(vrid.vrid)
+                    except Exception as e:
+                        vrid_summary = {}
+                        LOG.exception("Failed to get existing VRID summary due to: %s", str(e))
+
+                    if vrid_summary and 'floating-ip' in vrid_summary['vrid']:
+                        vrid_fip = vrid_summary['vrid']['floating-ip']
+                        if vthunder.partition_name != 'shared':
+                            for i in range(len(vrid_fip['ip-address-part-cfg'])):
+                                existing_fips.append(
+                                    vrid_fip['ip-address-part-cfg'][i]['ip-address-partition'])
+                        else:
+                            for i in range(len(vrid_fip['ip-address-cfg'])):
+                                existing_fips.append(vrid_fip['ip-address-cfg'][i]['ip-address'])
+
                     subnet_matched = list(filter(lambda x: x == vrid.subnet_id,
                                           subnet_list))
                     if subnet_matched:
                         vrids.append(vrid)
                         subnet = self.network_driver.get_subnet(vrid.subnet_id)
                         self.network_driver.deallocate_vrid_fip(vrid, subnet, amphorae)
-                    else:
+                    elif vrid.vrid_floating_ip in existing_fips:
                         vrid_floating_ip_list.append(vrid.vrid_floating_ip)
                 if not vthunder.partition_name or vthunder.partition_name == 'shared':
                     self.axapi_client.vrrpa.update(
@@ -940,13 +1006,13 @@ class ReserveSubnetAddressForMember(BaseNetworkTask):
 
         if nat_pool is None:
             try:
-                addr_list = []
-                start = (struct.unpack(">L", socket.inet_aton(nat_flavor['start_address'])))[0]
-                end = (struct.unpack(">L", socket.inet_aton(nat_flavor['end_address'])))[0]
-                while start <= end:
-                    addr_list.append(socket.inet_ntoa(struct.pack(">L", start)))
-                    start += 1
-                port = self.network_driver.reserve_subnet_addresses(member.subnet_id, addr_list)
+                addr_list = a10_utils.get_natpool_addr_list(nat_flavor)
+                if not CONF.vthunder.slb_no_snat_support:
+                    amphorae = a10_task_utils.attribute_search(member, 'amphorae')
+                else:
+                    amphorae = None
+                port = self.network_driver.reserve_subnet_addresses(
+                    member.subnet_id, addr_list, amphorae)
                 LOG.debug("Successfully allocated addresses for nat pool %s on port %s",
                           nat_flavor['pool_name'], port.id)
                 return port
@@ -971,7 +1037,56 @@ class ReleaseSubnetAddressForMember(BaseNetworkTask):
         if nat_pool.member_ref_count == 1:
             try:
                 self.network_driver.delete_port(nat_pool.port_id)
+                if not CONF.vthunder.slb_no_snat_support:
+                    addr_list = a10_utils.get_natpool_addr_list(nat_flavor)
+                    amphorae = a10_task_utils.attribute_search(member, 'amphorae')
+                    if amphorae is not None:
+                        self.network_driver.release_subnet_addresses(
+                            member.subnet_id, addr_list, amphorae)
             except Exception as e:
                 LOG.exception("Failed to release addresses in NAT pool %s from subnet %s",
                               nat_flavor['pool_name'], member.subnet_id)
                 raise e
+
+
+class GetMembersOnThunder(BaseNetworkTask):
+
+    @axapi_client_decorator
+    def execute(self, vthunder, use_device_flavor):
+        if vthunder and use_device_flavor:
+            try:
+                member_list = []
+                members = []
+                member_list = self.axapi_client.slb.server.get_all()
+                if member_list:
+                    for member in range(len(member_list['server-list'])):
+                        members.append(member_list['server-list'][member]['host'])
+                return members
+            except Exception as e:
+                LOG.exception("Failed to get members on the vthunder due to %s ",
+                              str(e))
+                raise e
+        else:
+            return
+
+
+class GetPoolsOnThunder(BaseNetworkTask):
+
+    @axapi_client_decorator
+    def execute(self, vthunder, use_device_flavor):
+        if vthunder and use_device_flavor:
+            try:
+                server_group_list = []
+                server_groups = []
+                server_group_list = self.axapi_client.slb.service_group.all()
+                if server_group_list:
+                    for server_group in range(len(server_group_list['service-group-list'])):
+                        server_groups.append(
+                            server_group_list['service-group-list'][server_group]['name'])
+                return server_groups
+            except Exception as e:
+                LOG.exception("Failed to get pools on the vthunder due to %s ",
+                              str(e))
+                raise e
+        else:
+            return
