@@ -40,6 +40,7 @@ from a10_octavia.controller.worker.tasks.decorators import activate_partition
 from a10_octavia.controller.worker.tasks.decorators import axapi_client_decorator
 from a10_octavia.controller.worker.tasks.decorators import axapi_client_decorator_for_revert
 from a10_octavia.controller.worker.tasks.decorators import device_context_switch_decorator
+from a10_octavia.controller.worker.tasks import utils as a10_task_utils
 from a10_octavia.db import repositories as a10_repo
 
 
@@ -71,25 +72,22 @@ class VThunderComputeConnectivityWait(VThunderBaseTask):
             if vthunder:
                 axapi_client = a10_utils.get_axapi_client(vthunder)
                 LOG.info("Attempting to connect vThunder device for connection.")
-                attempts = 30
-                # TODO(ytsai): use another new config option for vthunder booting wait retry
-                if CONF.a10_controller_worker.amp_active_retries > attempts:
-                    attempts = CONF.a10_controller_worker.amp_active_retries
+                attempts = CONF.a10_controller_worker.amp_active_retries
                 while attempts >= 0:
                     try:
                         attempts = attempts - 1
                         axapi_client.system.information()
                         break
-                    except (req_exceptions.ConnectionError, acos_errors.ACOSException,
-                            http_client.BadStatusLine, req_exceptions.ReadTimeout):
-                        attemptid = 21 - attempts
-                        sleep_time = 20
-                        # TODO(ytsai): use another new config option for vthunder booting wait sec
-                        if CONF.a10_controller_worker.amp_active_wait_sec > sleep_time:
-                            sleep_time = CONF.a10_controller_worker.amp_active_wait_sec
-                        time.sleep(sleep_time)
-                        LOG.debug("VThunder connection attempt - " + str(attemptid))
-                        pass
+                    except a10_task_utils.thunder_busy_exceptions():
+                        # acos-client already wait default_axapi_timeout for these exceptions.
+                        axapi_timeout = CONF.vthunder.default_axapi_timeout
+                        attempt_wait = CONF.a10_controller_worker.amp_active_wait_sec
+                        if axapi_timeout > attempt_wait:
+                            attempts = attempts - (axapi_timeout / attempt_wait - 1)
+                        LOG.debug("VThunder connection retry cnt:" + str(attempts))
+                    except (acos_errors.ACOSException, http_client.BadStatusLine):
+                        time.sleep(CONF.a10_controller_worker.amp_active_wait_sec)
+                        LOG.debug("VThunder connection retry cnt:" + str(attempts))
                 if attempts < 0:
                     LOG.error("Failed to connect vThunder in expected amount of boot time: %s",
                               vthunder.id)
@@ -129,9 +127,47 @@ class AmphoraePostVIPPlug(VThunderBaseTask):
 class AllowL2DSR(VThunderBaseTask):
     """Task to add wildcat address in allowed_address_pair for L2DSR"""
 
-    def execute(self, subnet, amphora):
+    def execute(self, subnet, amphora, lb_count_flavor, flavor_data=None):
         if CONF.vthunder.l2dsr_support:
-            self.network_driver.allow_use_any_source_ip_on_egress(subnet.network_id, amphora)
+            for amp in amphora:
+                self.network_driver.allow_use_any_source_ip_on_egress(subnet.network_id, amp)
+
+        if flavor_data:
+            deployment = flavor_data.get('deployment')
+            if deployment and 'dsr_type' in deployment:
+                if deployment['dsr_type'] == "l2dsr_transparent":
+                    for amp in amphora:
+                        self.network_driver.allow_use_any_source_ip_on_egress(
+                            subnet.network_id, amp)
+
+    def revert(self, subnet, amphora, lb_count_flavor, flavor_data=None, *args, **kwargs):
+        if lb_count_flavor > 1:
+            return
+
+        if flavor_data:
+            deployment = flavor_data.get('deployment')
+            if deployment and 'dsr_type' in deployment:
+                if deployment['dsr_type'] == "l2dsr_transparent":
+                    for amp in amphora:
+                        self.network_driver.remove_any_source_ip_on_egress(subnet.network_id, amp)
+
+
+class DeleteL2DSR(VThunderBaseTask):
+    """Task to delete wildcat address in allowed_address_pair for L2DSR"""
+
+    def execute(self, subnet, amphora, lb_count_flavor, lb_count_subnet, flavor_data=None):
+        if CONF.vthunder.l2dsr_support:
+            if lb_count_subnet <= 1:
+                for amp in amphora:
+                    self.network_driver.remove_any_source_ip_on_egress(subnet.network_id, amp)
+        else:
+            if lb_count_flavor <= 1 and flavor_data:
+                deployment = flavor_data.get('deployment')
+                if deployment and 'dsr_type' in deployment:
+                    if deployment['dsr_type'] == "l2dsr_transparent":
+                        for amp in amphora:
+                            self.network_driver.remove_any_source_ip_on_egress(
+                                subnet.network_id, amp)
 
 
 class AllowLoadbalancerForwardWithAnySource(VThunderBaseTask):
@@ -269,38 +305,17 @@ class ConfigureVRID(VThunderBaseTask):
     """Task to configure vThunder VRID"""
 
     @axapi_client_decorator
-    def execute(self, vthunder, vrid=1):
+    def execute(self, vthunder):
         try:
-            self.axapi_client.system.action.configureVRID(vrid)
+            vrid = CONF.a10_global.vrid
+            if not self.axapi_client.vrrpa.exists(vrid):
+                self.axapi_client.system.action.configureVRID(vrid)
             LOG.debug("Configured the master vThunder for VRID")
         except (acos_errors.ACOSException, req_exceptions.ConnectionError) as e:
             LOG.exception("Failed to configure VRID on vthunder: %s", str(e))
             raise e
 
 
-class ConfigureVRRPSync(VThunderBaseTask):
-    """Task to sync vThunder VRRP"""
-
-    @axapi_client_decorator
-    def execute(self, vthunder, backup_vthunder):
-        """Execute to sync up vrrp in two vThunder devices."""
-        attempts = 3
-        while attempts >= 0:
-            try:
-                attempts = attempts - 1
-                self.axapi_client.system.action.configSynch(backup_vthunder.ip_address,
-                                                            backup_vthunder.username,
-                                                            backup_vthunder.password)
-                LOG.debug("Waiting 30 seconds for config synch.")
-                time.sleep(30)
-                LOG.debug("Sync up for vThunder master")
-                break
-            except (acos_errors.ACOSException, req_exceptions.ConnectionError) as e:
-                if attempts < 0:
-                    LOG.exception("Failed VRRP sync: %s", str(e))
-                    raise e
-
-
 def configure_avcs(axapi_client, device_id, device_priority, floating_ip, floating_ip_mask):
     axapi_client.system.action.set_vcs_device(device_id, device_priority)
     axapi_client.system.action.set_vcs_para(floating_ip, floating_ip_mask)
@@ -341,10 +356,12 @@ class ConfigureaVCSBackup(VThunderBaseTask):
                     attempts = 0
                     LOG.debug("Configured the backup vThunder for aVCS: %s", vthunder.id)
                     break
-                except req_exceptions.ReadTimeout as e:
-                    # Don't retry for ReadTimout, since acos-client already already have
-                    # tries and timeout for axapi request. And it will take very long to
-                    # response this error.
+                except a10_task_utils.thunder_busy_exceptions() as e:
+                    # acos-client already wait default_axapi_timeout for these exceptions.
+                    axapi_timeout = CONF.vthunder.default_axapi_timeout
+                    attempt_wait = CONF.a10_controller_worker.amp_vcs_wait_sec
+                    if axapi_timeout > attempt_wait:
+                        attempts = attempts - (axapi_timeout / attempt_wait - 1)
                     if attempts < 0:
                         raise e
                 except Exception as e:
@@ -431,23 +448,60 @@ class ConfirmVRRPStatus(VThunderBaseTask):
 class HandleACOSPartitionChange(VThunderBaseTask):
     """Task to switch to specified partition"""
 
-    def execute(self, vthunder):
-        axapi_client = a10_utils.get_axapi_client(vthunder)
+    def _get_hmt_partition_name(self, loadbalancer):
+        partition_name = loadbalancer.project_id[:14]
+        if CONF.a10_global.use_parent_partition:
+            parent_project_id = a10_utils.get_parent_project(loadbalancer.project_id)
+            if parent_project_id:
+                if parent_project_id != 'default':
+                    partition_name = parent_project_id[:14]
+            else:
+                LOG.error(
+                    "The parent project for project %s does not exist. ",
+                    loadbalancer.project_id)
+                raise exceptions.ParentProjectNotFound(loadbalancer.project_id)
+        else:
+            LOG.warning(
+                "Hierarchical multitenancy is disabled, use_parent_partition "
+                "configuration will not be applied for loadbalancer: %s",
+                loadbalancer.id)
+        return partition_name
+
+    def execute(self, loadbalancer, vthunder_config):
+        axapi_client = a10_utils.get_axapi_client(vthunder_config)
+
+        partition_name = vthunder_config.partition_name
+        hierarchical_mt = vthunder_config.hierarchical_multitenancy
+        if hierarchical_mt == 'enable':
+            partition_name = self._get_hmt_partition_name(loadbalancer)
+
         try:
-            partition = axapi_client.system.partition.get(vthunder.partition_name)
+            partition = axapi_client.system.partition.get(partition_name)
+            partition_name = partition['partition-name']
         except acos_errors.NotFound:
             partition = None
 
+        # Check if partition is using old n-lbaas length
+        if partition is None and hierarchical_mt == 'enable':
+            try:
+                partition = axapi_client.system.partition.get(partition_name[:13])
+                partition_name = partition['partition-name']
+            except acos_errors.NotFound:
+                partition = None
+
         if partition is None:
             try:
-                axapi_client.system.partition.create(vthunder.partition_name)
+                axapi_client.system.partition.create(partition_name)
                 axapi_client.system.action.write_memory(partition="shared")
-                LOG.info("Partition %s created", vthunder.partition_name)
+                LOG.info("Partition %s created", partition_name)
             except (acos_errors.ACOSException, req_exceptions.ConnectionError) as e:
                 LOG.exception("Failed to create parition on vThunder: %s", str(e))
                 raise e
         elif partition.get("status") == "Not-Active":
-            raise exceptions.PartitionNotActiveError(partition, vthunder.ip_address)
+            raise exceptions.PartitionNotActiveError(partition, vthunder_config.ip_address)
+
+        vthunder_config.partition_name = partition_name
+        return vthunder_config
 
 
 class SetupDeviceNetworkMap(VThunderBaseTask):
@@ -459,8 +513,15 @@ class SetupDeviceNetworkMap(VThunderBaseTask):
 
     @axapi_client_decorator
     def execute(self, vthunder):
-        if vthunder and vthunder.project_id in CONF.hardware_thunder.devices:
-            vthunder_conf = CONF.hardware_thunder.devices[vthunder.project_id]
+        if not vthunder:
+            return
+        device_name = ''.join([a10constants.DEVICE_KEY_PREFIX,
+                               vthunder.device_name]) if vthunder.device_name else ''
+        vthunder_conf = None
+        if isinstance(CONF.hardware_thunder.devices, dict):
+            vthunder_conf = (CONF.hardware_thunder.devices.get(device_name) or
+                             CONF.hardware_thunder.devices.get(vthunder.project_id))
+        if vthunder_conf is not None:
             device_network_map = vthunder_conf.device_network_map
 
             # Case when device network map is not provided/length is 0
@@ -1059,7 +1120,7 @@ class VCSSyncWait(VThunderBaseTask):
 
     @axapi_client_decorator
     def execute(self, vthunder):
-        if CONF.a10_controller_worker.loadbalancer_topology != "ACTIVE_STANDBY":
+        if not vthunder or CONF.a10_controller_worker.loadbalancer_topology != "ACTIVE_STANDBY":
             return
 
         attempts = CONF.a10_controller_worker.amp_vcs_retries
@@ -1086,10 +1147,14 @@ class VCSSyncWait(VThunderBaseTask):
                             msg="vBlade not found in vcs-summary")
                     raise acos_errors.AxapiJsonFormatError(
                         msg="vMaster not found in vcs-summary")
-            except req_exceptions.ReadTimeout as e:
-                # Don't retry for ReadTimout, since acos-client already already have
-                # tries and timeout for axapi request. And it will take very long to
-                # response this error.
+                if vmaster_ready is True and vblade_ready is True:
+                    break
+            except a10_task_utils.thunder_busy_exceptions() as e:
+                # acos-client already wait default_axapi_timeout for these exceptions.
+                axapi_timeout = CONF.vthunder.default_axapi_timeout
+                attempt_wait = CONF.a10_controller_worker.amp_vcs_wait_sec
+                if axapi_timeout > attempt_wait:
+                    attempts = attempts - (axapi_timeout / attempt_wait - 1)
                 if attempts < 0:
                     LOG.exception("Failed to connect VCS device: %s", str(e))
                     raise e
@@ -1105,34 +1170,39 @@ class GetMasterVThunder(VThunderBaseTask):
 
     @axapi_client_decorator
     def execute(self, vthunder):
-        attempts = CONF.a10_controller_worker.amp_vcs_retries
-        while attempts >= 0:
-            try:
-                attempts = attempts - 1
-                vcs_summary = {}
-                vcs_summary = self.axapi_client.system.action.get_vcs_summary_oper()
-                vcs_member_list = vcs_summary['vcs-summary']['oper']['member-list']
-                for i in range(len(vcs_member_list)):
-                    role = vcs_member_list[i]['state'].split('(')[0]
-                    if role == "vMaster":
-                        vthunder.ip_address = vcs_member_list[i]['ip-list'][0]['ip']
-                        break
-                else:
-                    raise acos_errors.AxapiJsonFormatError(
-                        msg="vMaster not found in vcs-summary")
-                return vthunder
-            except req_exceptions.ReadTimeout as e:
-                # Don't retry for ReadTimout, since acos-client already already have
-                # tries and timeout for axapi request. And it will take very long to
-                # response this error.
-                if attempts < 0:
-                    LOG.exception("Failed to get Master vThunder: %s", str(e))
-                    raise e
-            except Exception as e:
-                if attempts < 0:
-                    LOG.exception("Failed to get Master vThunder: %s", str(e))
-                    raise e
-                time.sleep(CONF.a10_controller_worker.amp_vcs_wait_sec)
+        if vthunder:
+            attempts = CONF.a10_controller_worker.amp_vcs_retries
+            while attempts >= 0:
+                try:
+                    attempts = attempts - 1
+                    vcs_summary = {}
+                    vcs_summary = self.axapi_client.system.action.get_vcs_summary_oper()
+                    vcs_member_list = vcs_summary['vcs-summary']['oper']['member-list']
+                    for i in range(len(vcs_member_list)):
+                        role = vcs_member_list[i]['state'].split('(')[0]
+                        if role == "vMaster":
+                            vthunder.ip_address = vcs_member_list[i]['ip-list'][0]['ip']
+                            break
+                    else:
+                        raise acos_errors.AxapiJsonFormatError(
+                            msg="vMaster not found in vcs-summary")
+                    return vthunder
+                except a10_task_utils.thunder_busy_exceptions() as e:
+                    # acos-client already wait default_axapi_timeout for these exceptions.
+                    axapi_timeout = CONF.vthunder.default_axapi_timeout
+                    attempt_wait = CONF.a10_controller_worker.amp_vcs_wait_sec
+                    if axapi_timeout > attempt_wait:
+                        attempts = attempts - (axapi_timeout / attempt_wait - 1)
+                    if attempts < 0:
+                        LOG.exception("Failed to get Master vThunder: %s", str(e))
+                        raise e
+                except Exception as e:
+                    if attempts < 0:
+                        LOG.exception("Failed to get Master vThunder: %s", str(e))
+                        raise e
+                    time.sleep(CONF.a10_controller_worker.amp_vcs_wait_sec)
+        else:
+            return None
 
 
 class VthunderInstanceBusy(VThunderBaseTask):
@@ -1140,3 +1210,29 @@ class VthunderInstanceBusy(VThunderBaseTask):
     def execute(self, compute_busy=False):
         if compute_busy:
             raise Exception('vThunder instance is busy now, try again later.')
+
+
+class GetVthunderConfByFlavor(VThunderBaseTask):
+
+    def execute(self, loadbalancer, vthunder_config, device_config_dict, flavor_data=None,
+                flow_type=None):
+        if flavor_data is None and vthunder_config is None:
+            raise exceptions.ProjectDeviceNotFound()
+
+        if flavor_data is not None:
+            device_flavor = flavor_data.get('device_name', None)
+            if device_flavor is not None:
+                dev_key = a10constants.DEVICE_KEY_PREFIX + device_flavor
+                if dev_key in device_config_dict:
+                    vthunder_config = device_config_dict[dev_key]
+                    if flow_type is None or flow_type == a10constants.CREATE_FLOW:
+                        if vthunder_config.project_id is not None:
+                            raise exceptions.DeviceIsProjectDevice()
+                    vthunder_config.project_id = loadbalancer.project_id
+                    if vthunder_config.hierarchical_multitenancy == "enable":
+                        vthunder_config.partition_name = loadbalancer.project_id[0:14]
+                    return vthunder_config, True
+                else:
+                    raise exceptions.FlavorDeviceNotFound(device_flavor)
+
+        return vthunder_config, False
