@@ -41,6 +41,44 @@ SUBNET_1 = o_net_data_models.Subnet(id=a10constants.MOCK_SUBNET_ID)
 VRID_1 = data_models.VRID(id=1, subnet_id=a10constants.MOCK_SUBNET_ID)
 NAT_POOL = data_models.NATPool(port_id=a10constants.MOCK_PORT_ID)
 NAT_FLAVOR = {"pool_name": "p1", "start_address": "1.1.1.1", "end_address": "1.1.1.2"}
+HW_THUNDER = data_models.HardwareThunder(
+    project_id=a10constants.MOCK_PROJECT_ID,
+    device_name="rack_thunder_1",
+    undercloud=True,
+    username="abc",
+    password="abc",
+    ip_address="10.10.10.10",
+    partition_name="shared")
+HW_THUNDER2 = data_models.HardwareThunder(
+    project_id=a10constants.MOCK_PROJECT_ID,
+    device_name="rack_thunder_2",
+    undercloud=True,
+    username="abc",
+    password="abc",
+    ip_address="10.10.10.11",
+    partition_name="shared",
+    vrid_floating_ip="192.168.8.126")
+EXISTING_FIP_SHARED_PARTITION = {
+    u'vrid': {u'blade-parameters': {
+        u'priority': 150, u'uuid': u'41e54b26-bc4f-11eb-bd71-525400895118',
+        u'a10-url': u'/axapi/v3/vrrp-a/vrid/0/blade-parameters'},
+        u'uuid': u'41e5439c-bc4f-11eb-bd71-525400895118', u'floating-ip': {
+            u'ip-address-cfg': [{
+                u'ip-address': u'192.168.8.140'}, {u'ip-address': u'192.168.9.140'}]},
+            u'vrid-val': 0, u'preempt-mode': {u'threshold': 0, u'disable': 0},
+            u'a10-url': u'/axapi/v3/vrrp-a/vrid/0'}
+}
+EXISTING_FIP_L3V_PARTITION = {
+    u'vrid': {u'blade-parameters': {
+        u'priority': 150, u'uuid': u'41e54b26-bc4f-11eb-bd71-525400895118',
+        u'a10-url': u'/axapi/v3/vrrp-a/vrid/0/blade-parameters'},
+        u'uuid': u'41e5439c-bc4f-11eb-bd71-525400895118', u'floating-ip': {
+            u'ip-address-part-cfg': [{
+                u'ip-address-partition': u'192.168.8.140'},
+                {u'ip-address-partition': u'192.168.9.140'}]},
+            u'vrid-val': 0, u'preempt-mode': {u'threshold': 0, u'disable': 0},
+            u'a10-url': u'/axapi/v3/vrrp-a/vrid/0'}
+}
 
 
 class MockIP(object):
@@ -74,7 +112,8 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task.axapi_client = self.client_mock
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
-        result = mock_network_task.execute(VTHUNDER, MEMBER, [], subnet)
+        vthunder_config = copy.deepcopy(HW_THUNDER)
+        result = mock_network_task.execute(VTHUNDER, MEMBER, [], subnet, vthunder_config)
         self.assertEqual(result, [])
 
     @mock.patch('a10_octavia.common.utils.get_vrid_floating_ip_for_project',
@@ -87,15 +126,17 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         member.subnet_id = SUBNET_1.id
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         port = copy.deepcopy(PORT)
         port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FLOATING_IP_1))
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_SHARED_PARTITION
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         self.network_driver_mock.allocate_vrid_fip.return_value = port
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
-        mock_network_task.execute(VTHUNDER, member, [], subnet)
+        mock_network_task.execute(VTHUNDER, member, [], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_called_with(
             mock.ANY, None, mock.ANY,
             fixed_ip=a10constants.MOCK_VRID_FLOATING_IP_1)
@@ -103,6 +144,31 @@ class TestNetworkTasks(base.BaseTaskTestCase):
             VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FLOATING_IP_1],
             is_partition=False)
 
+    @mock.patch('a10_octavia.common.utils.get_patched_ip_address',
+                return_value=a10constants.MOCK_VRID_FLOATING_IP_1)
+    def test_HandleVRIDFloatingIP_create_floating_ip_with_device_name_flavor(
+            self, mock_patched_ip):
+        member = copy.deepcopy(MEMBER)
+        member.subnet_id = SUBNET_1.id
+        subnet = copy.deepcopy(SUBNET_1)
+        subnet.cidr = a10constants.MOCK_SUBNET_CIDR
+        vthunder_config = copy.deepcopy(HW_THUNDER2)
+        port = copy.deepcopy(PORT)
+        port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FLOATING_IP_1))
+        mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
+        mock_network_task.axapi_client = self.client_mock
+        self.network_driver_mock.get_subnet.return_value = subnet
+        self.network_driver_mock.allocate_vrid_fip.return_value = port
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_SHARED_PARTITION
+        self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
+                         vrid=VRID_VALUE)
+        mock_network_task.execute(VTHUNDER, member, [], subnet,
+                                  vthunder_config, use_device_flavor=True)
+        mock_network_task.axapi_client.get_vrid_floating_ip_for_project.assert_not_called()
+        self.client_mock.vrrpa.update.assert_called_with(
+            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FLOATING_IP_1],
+            is_partition=False)
+
     @mock.patch('a10_octavia.common.utils.get_vrid_floating_ip_for_project',
                 return_value=a10constants.MOCK_VRID_FLOATING_IP_1)
     @mock.patch('a10_octavia.common.utils.get_patched_ip_address',
@@ -114,6 +180,7 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         member.subnet_id = SUBNET_1.id
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         port = copy.deepcopy(PORT)
         port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FLOATING_IP_1))
         vthunder = copy.deepcopy(VTHUNDER)
@@ -122,9 +189,10 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         self.network_driver_mock.allocate_vrid_fip.return_value = port
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_L3V_PARTITION
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
-        mock_network_task.execute(vthunder, member, [], subnet)
+        mock_network_task.execute(vthunder, member, [], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_called_with(
             mock.ANY, None, mock.ANY,
             fixed_ip=a10constants.MOCK_VRID_FLOATING_IP_1)
@@ -144,15 +212,17 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         member.subnet_id = SUBNET_1.id
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         port = copy.deepcopy(PORT)
         port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FLOATING_IP_1))
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         self.network_driver_mock.allocate_vrid_fip.return_value = port
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_SHARED_PARTITION
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
-        mock_network_task.execute(VTHUNDER, member, [VRID_1], subnet)
+        mock_network_task.execute(VTHUNDER, member, [VRID_1], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_called_with(
             mock.ANY, None, None, fixed_ip=None)
         self.client_mock.vrrpa.update.assert_called_with(
@@ -171,6 +241,7 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         member.subnet_id = SUBNET_1.id
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         port = copy.deepcopy(PORT)
         port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FLOATING_IP_1))
         vthunder = copy.deepcopy(VTHUNDER)
@@ -179,9 +250,10 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         self.network_driver_mock.allocate_vrid_fip.return_value = port
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_L3V_PARTITION
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
-        mock_network_task.execute(vthunder, member, [VRID_1], subnet)
+        mock_network_task.execute(vthunder, member, [VRID_1], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_called_with(
             mock.ANY, None, None, fixed_ip=None)
         self.client_mock.vrrpa.update.assert_called_with(
@@ -196,9 +268,10 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         vrid = copy.deepcopy(VRID_1)
         vrid.vrid_port_id = a10constants.MOCK_VRRP_PORT_ID
         vrid.vrid = VRID_VALUE
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
-        result = mock_network_task.execute(VTHUNDER, MEMBER, [vrid], SUBNET_1)
+        result = mock_network_task.execute(VTHUNDER, MEMBER, [vrid], SUBNET_1, vthunder_config)
         self.network_driver_mock.delete_port.assert_called_with(
             vrid.vrid_port_id)
         self.client_mock.vrrpa.update.assert_called_with(
@@ -219,11 +292,13 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
         port = copy.deepcopy(PORT)
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FLOATING_IP_1))
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        mock_network_task.execute(VTHUNDER, member, [vrid], subnet)
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_SHARED_PARTITION
+        mock_network_task.execute(VTHUNDER, member, [vrid], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_not_called()
         self.client_mock.vrrpa.update.assert_not_called()
 
@@ -243,14 +318,16 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
         port = copy.deepcopy(PORT)
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FLOATING_IP_2))
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         self.network_driver_mock.allocate_vrid_fip.return_value = port
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_SHARED_PARTITION
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
-        mock_network_task.execute(VTHUNDER, member, [vrid], subnet)
+        mock_network_task.execute(VTHUNDER, member, [vrid], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_called_with(
             vrid, subnet.network_id, mock.ANY,
             fixed_ip=a10constants.MOCK_VRID_FLOATING_IP_2)
@@ -276,6 +353,7 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
         port = copy.deepcopy(PORT)
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FLOATING_IP_2))
         vthunder = copy.deepcopy(VTHUNDER)
         vthunder.partition_name = 'partition_1'
@@ -283,9 +361,10 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         self.network_driver_mock.allocate_vrid_fip.return_value = port
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_L3V_PARTITION
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
-        mock_network_task.execute(vthunder, member, [vrid], subnet)
+        mock_network_task.execute(vthunder, member, [vrid], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_called_with(
             vrid, None, mock.ANY,
             fixed_ip=a10constants.MOCK_VRID_FLOATING_IP_2)
@@ -308,11 +387,12 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         member.subnet_id = a10constants.MOCK_SUBNET_ID
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
-
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        fip_port = mock_network_task.execute(VTHUNDER, member, [vrid], subnet)
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_SHARED_PARTITION
+        fip_port = mock_network_task.execute(VTHUNDER, member, [vrid], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_not_called()
         self.network_driver_mock.delete_port.assert_not_called()
         self.assertEqual(fip_port, [vrid])
@@ -335,15 +415,17 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
         port = copy.deepcopy(PORT)
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FLOATING_IP_1))
 
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         self.network_driver_mock.allocate_vrid_fip.return_value = port
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_SHARED_PARTITION
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
-        mock_network_task.execute(VTHUNDER, member, [vrid], subnet)
+        mock_network_task.execute(VTHUNDER, member, [vrid], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_called_with(
             vrid, subnet.network_id, mock.ANY, fixed_ip=None)
         self.client_mock.vrrpa.update.assert_called_with(
@@ -368,6 +450,7 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
         port = copy.deepcopy(PORT)
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FLOATING_IP_1))
         vthunder = copy.deepcopy(VTHUNDER)
         vthunder.partition_name = 'partition_1'
@@ -375,9 +458,10 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         self.network_driver_mock.allocate_vrid_fip.return_value = port
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_L3V_PARTITION
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
-        mock_network_task.execute(vthunder, member, [vrid], subnet)
+        mock_network_task.execute(vthunder, member, [vrid], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_called_with(
             vrid, subnet.network_id, mock.ANY, fixed_ip=None)
         self.client_mock.vrrpa.update.assert_called_with(
@@ -396,15 +480,17 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
         port = copy.deepcopy(PORT)
+        vthunder_config = copy.deepcopy(HW_THUNDER)
         port.fixed_ips.append(MockIP(a10constants.MOCK_VRID_FULL_FLOATING_IP))
 
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         self.network_driver_mock.allocate_vrid_fip.return_value = port
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_SHARED_PARTITION
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
-        mock_network_task.execute(VTHUNDER, member, [VRID_1], subnet)
+        mock_network_task.execute(VTHUNDER, member, [VRID_1], subnet, vthunder_config)
         self.network_driver_mock.allocate_vrid_fip.assert_called_with(
             mock.ANY, subnet.network_id, mock.ANY,
             fixed_ip=a10constants.MOCK_VRID_FULL_FLOATING_IP)
@@ -419,8 +505,9 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         vrid.vrid_port_id = a10constants.MOCK_VRRP_PORT_ID
         vrid.vrid = VRID_VALUE
         mock_network_task.axapi_client = self.client_mock
-        result = mock_network_task.execute(VTHUNDER, [vrid], SUBNET_1,
-                                           0, 1, MEMBER)
+        self.client_mock.vrrpa.get.return_value = EXISTING_FIP_SHARED_PARTITION
+        result = mock_network_task.execute(VTHUNDER, [vrid], SUBNET_1, False,
+                                           0, 1, 0, 0, MEMBER)
         self.network_driver_mock.deallocate_vrid_fip.assert_called_with(
             vrid, mock.ANY, mock.ANY)
         self.client_mock.vrrpa.update.assert_called_with(
@@ -430,8 +517,8 @@ class TestNetworkTasks(base.BaseTaskTestCase):
     def test_DeleteMemberVRIDPort_member_count_lb_count(self):
         mock_network_task = a10_network_tasks.DeleteVRIDPort()
         mock_network_task.axapi_client = self.client_mock
-        result = mock_network_task.execute(VTHUNDER, [VRID_1], SUBNET_1,
-                                           1, 1, MEMBER)
+        result = mock_network_task.execute(VTHUNDER, [VRID_1], SUBNET_1, False,
+                                           1, 1, 0, 0, MEMBER)
         self.network_driver_mock.deallocate_vrid_fip.assert_not_called()
         self.client_mock.vrrpa.delete.assert_not_called()
         self.assertEqual(result, (None, False))
@@ -460,12 +547,13 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         self.client_mock.vrrpa.update.assert_not_called()
         self.assertEqual(result, None)
 
-    def test_reserve_subnet_addr_for_member(self):
+    @mock.patch('a10_octavia.controller.worker.tasks.a10_network_tasks.a10_task_utils')
+    def test_reserve_subnet_addr_for_member(self, mock_utils):
         mock_network_task = a10_network_tasks.ReserveSubnetAddressForMember()
         mock_network_task.network_driver = self.client_mock
         mock_network_task.execute(MEMBER, NAT_FLAVOR)
         self.client_mock.reserve_subnet_addresses.assert_called_with(
-            MEMBER.subnet_id, ["1.1.1.1", "1.1.1.2"])
+            MEMBER.subnet_id, ["1.1.1.1", "1.1.1.2"], mock.ANY)
 
     def test_release_subnet_addr_referenced(self):
         mock_network_task = a10_network_tasks.ReleaseSubnetAddressForMember()
@@ -473,7 +561,8 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         ret_val = mock_network_task.execute(MEMBER, NAT_FLAVOR, NAT_POOL)
         self.assertEqual(ret_val, None)
 
-    def test_release_subnet_addr(self):
+    @mock.patch('a10_octavia.controller.worker.tasks.a10_network_tasks.a10_task_utils')
+    def test_release_subnet_addr(self, mock_utils):
         mock_network_task = a10_network_tasks.ReleaseSubnetAddressForMember()
         mock_network_task.network_driver = self.client_mock
         NAT_POOL.member_ref_count = 1
