@@ -23,6 +23,8 @@ from octavia.common import data_models as o_data_models
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 
+from acos_client import errors as acos_errors
+
 from a10_octavia.common import config_options
 from a10_octavia.common.data_models import VThunder
 from a10_octavia.controller.worker.tasks import health_monitor_tasks as task
@@ -37,6 +39,7 @@ HM = o_data_models.HealthMonitor(id=a10constants.MOCK_HM_ID,
                                  timeout=3,
                                  rise_threshold=8,
                                  http_method='GET')
+
 ARGS = utils.meta(HM, 'hm', {})
 LISTENERS = [o_data_models.Listener(id=a10constants.MOCK_LISTENER_ID, protocol_port=mock.ANY)]
 
@@ -224,7 +227,37 @@ class TestHandlerHealthMonitorTasks(BaseTaskTestCase):
                                                           expect_code=None, post_data=None,
                                                           **FLAVOR_WITH_REGEX_ARGS)
 
-    def test_health_monitor_update_task(self):
+    def test_get_hm_name(self):
+        hm = {
+            'monitor': {
+                'name': a10constants.MOCK_HM_ID
+            }
+        }
+        self.client_mock.slb.hm.get.return_value = hm
+        hm_name = task._get_hm_name(self.client_mock, HM)
+        self.assertEqual(a10constants.MOCK_HM_ID, hm_name)
+
+    def test_get_hm_name_backwards_compat(self):
+        hm = {
+            'monitor': {
+                'name': a10constants.MOCK_HM_ID[0:28]
+            }
+        }
+        self.client_mock.slb.hm.get.return_value = hm
+        hm_name = task._get_hm_name(self.client_mock, HM)
+        self.assertEqual(a10constants.MOCK_HM_ID[0:28], hm_name)
+
+    def test_get_hm_name_raise_notfound(self):
+        self.client_mock.slb.hm.get.side_effect = acos_errors.NotFound
+
+        expected_error = acos_errors.NotFound
+        module_func = task._get_hm_name
+        func_args = [self.client_mock, HM]
+        self.assertRaises(expected_error, module_func, *func_args)
+
+    @mock.patch('a10_octavia.controller.worker.tasks.health_monitor_tasks._get_hm_name')
+    def test_health_monitor_update_task(self, mock_hm_name):
+        mock_hm_name.return_value = a10constants.MOCK_HM_ID
         hm_task = task.UpdateHealthMonitor()
         hm_task.axapi_client = self.client_mock
         update_dict = {'delay': '10'}
@@ -248,7 +281,9 @@ class TestHandlerHealthMonitorTasks(BaseTaskTestCase):
                                                           expect_code=None, post_data='abc=1',
                                                           **ARGS)
 
-    def test_health_monitor_update_with_flavor_and_config_task(self):
+    @mock.patch('a10_octavia.controller.worker.tasks.health_monitor_tasks._get_hm_name')
+    def test_health_monitor_update_with_flavor_and_config_task(self, mock_hm_name):
+        mock_hm_name.return_value = a10constants.MOCK_HM_ID
         flavor = {
             "health_monitor": {
                 "retry": 5,
@@ -275,7 +310,9 @@ class TestHandlerHealthMonitorTasks(BaseTaskTestCase):
                                                           post_data='abc=1',
                                                           expect_code=None, **FLAVOR_ARGS)
 
-    def test_health_monitor_update_with_flavor_task(self):
+    @mock.patch('a10_octavia.controller.worker.tasks.health_monitor_tasks._get_hm_name')
+    def test_health_monitor_update_with_flavor_task(self, mock_hm_name):
+        mock_hm_name.return_value = a10constants.MOCK_HM_ID
         flavor = {
             "health_monitor": {
                 "retry": 5,
@@ -300,7 +337,9 @@ class TestHandlerHealthMonitorTasks(BaseTaskTestCase):
                                                           port=mock.ANY, url=None, post_data=None,
                                                           expect_code=None, **FLAVOR_ARGS)
 
-    def test_health_monitor_update_with_flavor_regex_task(self):
+    @mock.patch('a10_octavia.controller.worker.tasks.health_monitor_tasks._get_hm_name')
+    def test_health_monitor_update_with_flavor_regex_task(self, mock_hm_name):
+        mock_hm_name.return_value = a10constants.MOCK_HM_ID
         flavor = {
             "health_monitor": {
                 "retry": 5,
@@ -339,7 +378,9 @@ class TestHandlerHealthMonitorTasks(BaseTaskTestCase):
                                                           expect_code=None, post_data=None,
                                                           **FLAVOR_WITH_REGEX_ARGS)
 
-    def test_health_monitor_update_with_regex_overwrite_flavor_task(self):
+    @mock.patch('a10_octavia.controller.worker.tasks.health_monitor_tasks._get_hm_name')
+    def test_health_monitor_update_with_regex_overwrite_flavor_task(self, mock_hm_name):
+        mock_hm_name.return_value = a10constants.MOCK_HM_ID
         flavor = {
             "health_monitor": {
                 "retry": 5,
@@ -379,7 +420,9 @@ class TestHandlerHealthMonitorTasks(BaseTaskTestCase):
                                                           expect_code=None, post_data=None,
                                                           **FLAVOR_WITH_REGEX_ARGS)
 
-    def test_health_monitor_delete_task(self):
+    @mock.patch('a10_octavia.controller.worker.tasks.health_monitor_tasks._get_hm_name')
+    def test_health_monitor_delete_task(self, mock_hm_name):
+        mock_hm_name.return_value = a10constants.MOCK_HM_ID
         hm_task = task.DeleteHealthMonitor()
         hm_task.axapi_client = self.client_mock
         mock_hm = copy.deepcopy(HM)
